# -*- coding: utf-8 -*-
# Copyright (C) 2023-2024 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
import logging

import os
import re

from tgp_backend.config import MAX_FILENAME_LENGTH
from tgp_backend.util import get_file_extension

log = logging.getLogger(__name__)


def generateList(path=None):
    path = path if path else os.getcwd()
    if not path.startswith("/"):
        path = "/%s" % path
    dList = []
    fList = []
    tp_dict = {}

    log.debug("generateList - path: %s" % path)

    for name in os.listdir(path):
        fullpath = "%s/%s" % (path, name)
        if not name.startswith("."):
            if os.path.isdir(fullpath):
                dList.append({"fullpath": fullpath, "name": name})
            else:
                fList.append({"fullpath": fullpath, "name": name})
    dir_list_dict = []
    file_list_dict = []
    # curDir = os.getcwd()
    curDir = path

    for d in dList:
        if len(d["name"]) > MAX_FILENAME_LENGTH:
            dots = "..."
        else:
            dots = ""
        temp_dir = {}
        temp_dir["f"] = d["name"][0:MAX_FILENAME_LENGTH] + dots
        temp_dir["name"] = d["name"]
        temp_dir["f_url"] = re.sub("#", "|HASHTAG|", d["fullpath"])
        temp_dir["currentDir"] = curDir
        temp_dir["f_complete"] = d["fullpath"]
        temp_dir["size"] = "---"

        dir_list_dict.append(temp_dir)

    for f in fList:
        image = None
        try:
            tp = get_file_extension(f["name"])
        except Exception as e:
            log.warn(e)
            pass
        if not image:
            image = "files_icon/unknown-icon.png"
        if len(f["name"]) > MAX_FILENAME_LENGTH:
            dots = "..."
        else:
            dots = ""
        temp_file = {}
        temp_file["f"] = f["name"][0:MAX_FILENAME_LENGTH] + dots
        temp_file["name"] = f["name"]
        temp_file["name_stripped"] = f["name"].replace(f".{tp}", "")
        temp_file["f_url"] = re.sub("#", "|HASHTAG|", f["fullpath"])
        temp_file["extension"] = tp.lower()
        temp_file["currentDir"] = curDir
        temp_file["fullpath"] = f["fullpath"]
        temp_file["image"] = image
        temp_file["is_xml"] = tp.lower() == "xml"

        file_list_dict.append(temp_file)

    return dir_list_dict, file_list_dict



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging
import nextcloud
import os

log = logging.getLogger(__name__)


class Nextcloud:
    def __init__(
        self,
        url=None,
        username=None,
        password=None,
        folder=None,
        **kwargs,
    ):
        self._nxc = None
        self._root = None
        self.url = url
        self.username = username
        self.password = password
        self.root_dir = folder

    def test_connection(self):
        if all([self.url, self.username, self.password]):
            return str(self.nxc.list_folders().status_code).startswith("2")
        return False

    @property
    def nxc(self):
        if self._nxc is None:
            self._nxc = nextcloud.NextCloud(
                endpoint=self.url,
                user=self.username,
                password=self.password,
            )
        return self._nxc

    @property
    def root(self):
        if self._root is None:
            log.debug("start get_folder")
            self._root = self.nxc.get_folder(self.root_dir)
            if self._root is None:
                raise "Could not find the Root Directory"
        return self._root

    def nxc_list_files_and_folders(self, get_selectable=False):

        selectable_folders = []

        def recursive_list(item, depth=0):
            items = []
            for d in item.list():
                if d.isdir():
                    children = recursive_list(d, depth=depth + 1)
                    contains_xml = any(
                        f.get("name", "").endswith(".xml") for f in children
                    )
                    item = {
                        "type": "folder",
                        "name": d.basename(),
                        "depth": depth,
                        "path": d.get_relative_path(),
                        "contains_xml": contains_xml,
                        "children": {
                            "count": len(children),
                            "list": children,
                        },
                    }
                    items.append(item)
                    if contains_xml:
                        selectable_folders.append(item)
                else:
                    items.append(
                        {
                            "type": "file",
                            "name": d.basename(),
                            "depth": depth,
                        }
                    )
            return items

        result = recursive_list(self.root)
        if get_selectable:
            return selectable_folders
        else:
            return result

    def get_selectable_folders(self):

        return self.nxc_list_files_and_folders(get_selectable=True)

    def download_nxc_files(self, file_paths, projectname):
        for path in file_paths:
            # Retrieve element (file or folder) from the server
            try:
                element = self.nxc.get_folder(path)
                log.debug(f"Processing {path} in project {projectname}")

                # Determine destination folder
                destination_folder = (
                    f"projects/{projectname}/data/{element.basename()}"
                )
                log.debug(f"Destination folder: {destination_folder}")

                if not os.path.exists(destination_folder):
                    os.makedirs(destination_folder)

                # Recursive function to download files and folders
                def download_recursive(
                    current_element, current_path, current_dest
                ):
                    for item in current_element.list():
                        item_path = os.path.join(current_path, item.basename())
                        if item.isdir():
                            # If it's a folder, create subfolder and continue
                            # recursively
                            new_dest = os.path.join(
                                current_dest, item.basename()
                            )
                            if not os.path.exists(new_dest):
                                os.makedirs(new_dest)
                            subfolder = self.nxc.get_folder(item_path)
                            download_recursive(subfolder, item_path, new_dest)
                        else:
                            # If it's a file, download it
                            remote_file = self.nxc.get_file(item_path)
                            target_path = os.path.join(
                                current_dest, item.basename()
                            )
                            remote_file.download(
                                target=target_path, overwrite=True
                            )
                            log.debug(f"Downloaded: {target_path}")

                # Start the recursive download
                download_recursive(element, path, destination_folder)

            except Exception as e:
                log.error(f"Error processing {path}: {str(e)}")



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de
import logging

import json
import os
import secrets

from typing import Optional

# Removed unused import of Path
from pathlib import Path

log = logging.getLogger(__name__)


class SecretKeyManager:
    """Manages the secret key for Flask sessions."""

    def __init__(self, base_path: str, env_var: str = "FLASK_SECRET_KEY"):
        # Initialize with the base path and environment variable name
        self.env_var = env_var
        self.secret_file = Path(base_path) / ".secret_key"
        self._secret_key: Optional[str] = None

    @property
    def secret_key(self) -> str:
        """Returns the secret key or generates a new one if it doesn't exist."""
        if self._secret_key is None:
            self._secret_key = self._get_or_create_key()
        return self._secret_key

    def _get_or_create_key(self) -> str:
        """Fetches the key from the environment or file, or creates a new one."""
        # 1. Try to get the key from the environment variable
        if env_key := os.environ.get(self.env_var):
            log.info(f"Using secret key from {self.env_var}")
            return env_key

        # 2. Try to load the key from the saved file
        try:
            if self.secret_file.exists():
                with open(self.secret_file) as f:
                    log.info(f"Loading secret key from {self.secret_file}")
                    return json.load(f)
        except Exception as e:
            log.warning(f"Could not load secret key: {e}")

        # 3. Generate a new key
        new_key = secrets.token_hex(32)
        log.info("Generating a new secret key")

        try:
            # Create the directory if it doesn't exist
            self.secret_file.parent.mkdir(parents=True, exist_ok=True)

            # Save the key to the file
            with open(self.secret_file, "w") as f:
                json.dump(new_key, f)

            # Set file permissions
            self.secret_file.chmod(0o600)

            log.info(f"New secret key saved to {self.secret_file}")
            return new_key

        except Exception as e:
            log.warning(f"Could not save secret key: {e}")
            return new_key



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2024 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
import logging

import os
import re

from collections import defaultdict

from .config import get_config_value

log = logging.getLogger(__name__)


def config(*args, **kwargs):
    log.warning(
        """
            Using deprecated config function.
            Please use tgp_backend.config.get_config_value instead.
        """
    )
    return get_config_value(*args, **kwargs)


def cli_startup(log_level=logging.INFO, log_file=None):
    log_config = dict(
        level=log_level,
        format="%(asctime)s %(name)-10s %(levelname)-4s %(message)s",
    )
    if log_file:
        log_config["filename"] = log_file

    logging.basicConfig(**log_config)
    logging.getLogger("").setLevel(log_level)


def get_file_extension(fname):
    found_extension = re.search("\.[A-Za-z0-9]*$", fname, re.IGNORECASE)
    if found_extension:
        return found_extension[0][1:].lower()
    return ""


def list_files_and_folders(path, get_selectable=False):

    selectable_folders = []

    def recursive_list(dir_path, depth=0):
        items = []
        for entry in os.scandir(dir_path):
            if entry.name.startswith("."):
                continue

            if entry.is_dir():
                children = recursive_list(entry.path, depth=depth + 1)
                contains_xml = any(
                    f.get("name", "").endswith(".xml") for f in children
                )
                item = {
                    "type": "folder",
                    "name": entry.name,
                    "depth": depth,
                    "path": entry.path,
                    "contains_xml": contains_xml,
                    "children": {"count": len(children), "list": children},
                }
                items.append(item)
                if contains_xml:
                    selectable_folders.append(item)
            else:
                items.append(
                    {
                        "type": "file",
                        "name": entry.name,
                        "depth": depth,
                        "path": entry.path,
                    }
                )
        return items

    result = recursive_list(path)
    if get_selectable:
        return selectable_folders
    else:
        return result


def get_selectable_folders(path):

    return list_files_and_folders(path, get_selectable=True)


def remove_empty_strings_from_dict(d):
    for key in d:
        if d[key] == "":
            d[key] = None
    return d


def parse_request_data(request, attrib_prefix):
    """
    Processes form data and creates a list of dictionaries
    for the specified attributes.
    """
    grouped_data = defaultdict(lambda: defaultdict(list))

    # Group data based on prefix
    for key in request.form.keys():
        if key.startswith(attrib_prefix):
            try:
                _, field, data_type = key.split("-")
                grouped_data[field][data_type] = request.form.getlist(key)
            except ValueError:
                log.warning(
                    f"Invalid key format: {key}. Expected format: '{attrib_prefix}-field-data_type'."
                )
                continue  # Skip invalid keys

    # Transform grouped data into desired structure
    max_length = max(len(data["xpath"]) for data in grouped_data.values())
    return [
        {
            field: {
                "xpath": data["xpath"][i],
                "value": data["value"][i],
            }
            for field, data in grouped_data.items()
        }
        for i in range(max_length)
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

import os
import shutil

from flask import Blueprint, render_template, request, send_file, session
from flask_json import json_response

from io import BytesIO

from tgp_backend.config import PATH_DATA
from tgp_backend.nextcloud import Nextcloud
from tgp_backend.project import Project
from tgp_backend.session_manager import SessionManagerNextcloud

log = logging.getLogger(__name__)

data_routes = Blueprint("data", __name__)


# ***TABS***
# **********
@data_routes.route("/tab-import-data/<string:projectname>", methods=["GET"])
def tab_import_data(projectname=None, project=None, error_message=None):
    if not project:
        if not projectname:
            raise ValueError("Project name must be provided.")
        else:
            project = Project(projectname)
    return render_template(
        "tabs/import_data.html",
        project=project,
        error_message=error_message,
    )


# ***DIRECT UPLOAD***
# *******************
@data_routes.route("/upload_files/<string:projectname>", methods=["POST"])
def upload_files(projectname=None):
    project = Project(projectname)
    project.file_upload(request.files.getlist("files"))

    return tab_import_data(project=project)


# ***GITHUB***
# ************
@data_routes.route("/modal/github/<projectname>", methods=["GET", "POST"])
def modal_github(projectname):
    """
    Render the Github modal.
    """
    project = Project(projectname)
    return render_template(
        "modal/github_modal.html",
        project=project,
    )


@data_routes.route("/clone_git_project/<string:projectname>", methods=["POST"])
def clone_git_project(projectname=None):
    project = Project(projectname)
    project.clone_git_project(request.form.get("github_repo"))

    return tab_import_data(project=project)


# ***NEXTCLOUD***
# ***************


def render_nextcloud_tree(projectname, items):
    """
    Render the Nextcloud tree view.
    """
    # items = items or nc.nxc_list_files_and_folders()
    project = Project(projectname)
    return render_template(
        "modal/file_explorer_nextcloud.html",
        project=project,
        items=items,
        show_checkbox=True,
        show_all_folders=True,
    )


def render_nextcloud_login(projectname, message=None):
    """
    Render the Nextcloud login form.
    """
    return render_template(
        "modal/nextcloud_login.html", projectname=projectname, message=message
    )


@data_routes.route("/modal/nextcloud/<projectname>", methods=["GET", "POST"])
@data_routes.route(
    "/modal/nextcloud/<projectname>/<logout>", methods=["GET", "POST"]
)
def modal_nextcloud(projectname, form_type=None, logout=False):

    nxc_session_manager = SessionManagerNextcloud(session)

    if logout:
        # If the user wants to log out, delete the session credentials
        nxc_session_manager.delete_nextcloud_credentials()
        return render_nextcloud_login(projectname, message="You logged out.")

    nc = Nextcloud(**nxc_session_manager.get_credentials())
    if nc.test_connection():
        # An active Nextcloud session exists, so we can use it to render
        # the tree view
        return render_nextcloud_tree(
            projectname, nc.nxc_list_files_and_folders()
        )
    elif nxc_session_manager.request_has_valid_credentials(request):
        # If credentials are provided,
        # Recreate a Nextcloud instance and test the connection
        nxc_session_manager.save_credentials(request.form)
        nc = Nextcloud(**nxc_session_manager.get_credentials())
        if nc.test_connection():
            return render_nextcloud_tree(
                projectname, nc.nxc_list_files_and_folders()
            )

        else:
            return render_nextcloud_login(
                projectname,
                message="Login failed. Please check your credentials.",
            )
    else:
        # If no valid credentials are provided, render the login form
        log.info("No valid Nextcloud credentials found, showing login form.")
        # This will clear any previous session data
        nxc_session_manager.delete_nextcloud_credentials()
        return render_nextcloud_login(projectname)


@data_routes.route("/download_nextcloud/<projectname>", methods=["POST"])
def download_nextcloud(projectname):

    nxc_session_manager = SessionManagerNextcloud(session)

    selected_option = request.form.getlist("selected_folder")

    if selected_option:
        nc = Nextcloud(**nxc_session_manager.get_credentials())
        nc.download_nxc_files(selected_option, projectname=projectname)

        return json_response(
            status=200,
            message="Downloaded.",
        )
    else:
        return json_response(
            status=400, message="No folder selected for download."
        )


# ***OTHER***
# *******************
@data_routes.route("/image/<path:filepath>")
def serve_image(filepath):
    filepath = "/" + filepath
    if os.path.isfile(filepath):
        return send_file(filepath)
    return "", 404


@data_routes.route("/file/<path:filepath>")
def serve_file(filepath):

    filepath = "/" + filepath

    if os.path.isfile(filepath):
        with open(filepath, "rb") as file:
            file_content = BytesIO(file.read())

        return send_file(
            file_content,
            download_name=os.path.basename(filepath),
            as_attachment=True,
        )


@data_routes.route("/api/delete-folder", methods=["DELETE"])
def delete_folder():
    error_message = None  # Variable für die Fehlermeldung
    try:
        path = request.form.get("path")
        projectname = request.form.get("projectname")

        if not path:
            error_message = "No path specified"
        elif not path.startswith(PATH_DATA):
            # Security check to ensure path is within allowed directory
            error_message = "Invalid path"
        elif not os.path.exists(path):
            # Check if path exists
            error_message = "Path not found"
        else:
            # Check if the path is a file or a folder
            if os.path.isfile(path):
                # Delete the file
                os.remove(path)
                log.info(f"File deleted: {path}")
            elif os.path.isdir(path):
                # Delete the folder and its contents
                shutil.rmtree(path)
                log.info(f"Folder deleted: {path}")
            else:
                error_message = "Path is neither a file nor a folder"

    except Exception as e:
        log.error(f"Error deleting path: {e}")
        error_message = f"An error occurred: {str(e)}"

    # Am Ende einmal `tab_import_data` aufrufen
    return tab_import_data(
        projectname=projectname,
        error_message=error_message,
    )



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de
import logging

import json
import os
import subprocess

from datetime import datetime

from tg_model.collection import CollectionModeler  # type: ignore
from tg_model.project import Project as ProjectHandler  # type: ignore
from tg_model.tei import TEIParser  # type: ignore

from tg_model.yaml import (  # type: ignore
    CollectionConfig,
    CollectionConfigTemplate,
    ProjectConfigTemplate,
    ProjectConfig,
)

from tgp_backend.config import PATH_PROJECTS, PATH_DATA
from tgp_backend.tgclient import TGclient
from tgp_backend.util import (
    list_files_and_folders,
    get_selectable_folders,
)

log = logging.getLogger(__name__)


class Project(object):
    def __init__(self, path):
        self.path = path
        self._project_config = None
        self._collections = None
        self._nextcloud = None

    @property
    def name(self):
        log.warning("Project.name is deprecated, use Project.path instead")
        return self.path

    @property
    def fullpath(self):
        log.warning("#" * 50)
        log.warning(
            "DEPRECATED: Use PATH_PROJECTS from config instead of Project.fullpath"
        )
        return os.path.join(PATH_PROJECTS, self.path)

    @property
    def project_path(self):
        return os.path.join(PATH_PROJECTS, self.path)

    @property
    def title(self):
        return self.project_config.title

    @property
    def description(self):
        return self.project_config.description

    @property
    def collectors(self):
        return self.project_config.collectors

    @property
    def path_data(self):
        return PATH_DATA

    @property
    def path_metadata(self):
        log.warning(
            "DEPRECATED: Use Project.project_path instead of Project.path_metadata"
        )
        return self.project_path

    @property
    def path_otherfiles(self):
        return os.path.join(self.project_path, "other_files")

    def create(self):
        if not os.path.exists(self.project_path):
            os.makedirs(self.project_path, exist_ok=True)
        else:
            log.warning("Project already exists!")

    @property
    def avatar(self):
        if self.project_config.avatar:
            return os.path.join(
                self.path_otherfiles, self.project_config.avatar
            )

    @property
    def xslt(self):
        if self.project_config.xslt:
            return os.path.join(self.path_otherfiles, self.project_config.xslt)

    def save_avatar(self, file):
        if file.filename and file.filename != "":
            file.save(os.path.join(self.path_otherfiles, file.filename))
            self.project_config.avatar = file.filename

    def save_xslt(self, file):
        if file.filename and file.filename != "":
            file.save(os.path.join(self.path_otherfiles, file.filename))
            self.project_config.xslt = file.filename

    def clear_xslt(self):
        self.project_config.xslt = None
        self.project_config.save()

    def update(self, title, description, collectors, avatar, xslt):
        self.project_config.title = title
        self.project_config.description = description
        self.project_config.collectors = collectors

        if avatar:
            self.save_avatar(avatar)
        self.save_xslt(xslt)

        self.project_config.save()

    def clone_git_project(self, url):
        repo_name = url.split("/")[-1].replace(".git", "")
        repo_path = os.path.join(self.path_data, repo_name)

        if not os.path.exists(repo_path):
            subprocess.run(["git", "clone", url, repo_path])
        else:
            log.info("Repository already exists!")

    def file_upload(self, files):
        for file in files:
            if file:
                filename = file.filename
                filepath = os.path.join(self.path_data, filename)
                directory = os.path.dirname(filepath)
                if not os.path.exists(directory):
                    os.makedirs(directory)
                file.save(filepath)

    def list_files_and_folders(self):
        return list_files_and_folders(self.path_data)

    def get_selectable_folders(self):
        selectable_folders = get_selectable_folders(self.path_data)
        for folder in selectable_folders:
            folder["relative_path"] = folder["path"].replace(
                self.path_data, ""
            )
        return selectable_folders

    @property
    def main_config(self):
        log.warning(
            "Project.main_config is deprecated, use Project.project_config instead"
        )
        return self.project_config

    @property
    def project_config(self):
        if not self._project_config:
            self._project_config = ProjectConfig(projectpath=self.project_path)
            if not self._project_config.exists():
                self._project_config = ProjectConfigTemplate(
                    self.project_path
                ).render()
                self._project_config.title = self.path
                self._project_config.save()

        return self._project_config

    @property
    def project_handler(self):
        return ProjectHandler(self.project_path)

    def get_subproject_inpath(self, name):
        for subproject in self.project_config.content["subprojects"]:
            if subproject["name"] == name:
                return subproject["inpath"]
        return None

    def _set_collections(self):
        self._collections = {}
        if self.project_config.exists():
            for subproject in self.project_config.content["subprojects"]:
                collection_config = CollectionConfig(subproject["basepath"])
                if not collection_config.exists():
                    collection_config = CollectionConfigTemplate(
                        projectpath=self.project_path,
                        subproject=subproject,
                        files=[
                            TEIParser(fullpath=file)
                            for file in subproject["files"]
                        ],
                    ).render(overwrite=False)

                c_modeler = CollectionModeler(subproject, self.project_path)
                self._collections[subproject["name"]] = {
                    "config": collection_config,
                    "paths": subproject,
                    "parser": CollectionParser(collection_config),
                    "modeler": c_modeler,
                    "path": c_modeler.get_collection_path(),
                }

    @property
    def collections(self):
        if self._collections is None:
            self._set_collections()
        return self._collections

    def get_collection_paths(self):
        return [c["path"] for c in self.collections.values()]

    def get_collection(self, title):
        return self.collections.get(title)

    def get_collection_parser(self, collection):
        return CollectionParser(collection)

    def get_tgp(self, instance):
        return TGProject(self, instance)

    def _save_validation_results(self, results, filepath):
        """
        Saves the validation results to a JSON file.
        Adds the date of the last execution.
        """
        results["last_run"] = datetime.now().strftime("%Y-%m-%d %H:%M")

        with open(filepath, "w", encoding="utf-8") as f:
            json.dump(results, f, indent=4, ensure_ascii=False)

        log.info(f"Validation results saved to {filepath}")

    def _run_validation(self):
        validation_results = self.project_handler.validate()
        self._save_validation_results(validation_results, self.validation_file)
        return validation_results

    @property
    def validation_file(self):
        """
        Returns the path to the validation results file.
        """
        return os.path.join(self.project_path, "validation_results.json")

    def get_validation_results(self, refresh=False):
        """
        Returns stored validation results.
        If no results are saved or `refresh=True`, a new validation will
        be performed.
        """

        # If `refresh` is set or file doesn't exist, perform new validation
        if refresh or not os.path.exists(self.validation_file):
            validation_results = self._run_validation()
        else:
            # Load results from file
            with open(self.validation_file, "r", encoding="utf-8") as f:
                validation_results = json.load(f)

        if "ready_for_publication" not in validation_results:
            validation_results = self._run_validation()
        return validation_results


class TGProject(object):

    def __init__(self, project, instance):
        if isinstance(project, Project):
            self.project = project
        else:
            raise ValueError(
                "Project must be an instance of Project or a project path."
            )
        self.instance = instance
        self.project_config = self.project.project_config
        self.collections = self.project.collections
        self._tg_client = None
        self._tg_session_id = None
        self._tg_project_id = None

    def check_session(self):
        return self.tg_client.check_session()

    @property
    def tg_session_id(self):
        if not self._tg_session_id:
            self._tg_session_id = self.project_config.get_tg_session_id(
                self.instance
            )
        return self._tg_session_id

    @tg_session_id.setter
    def tg_session_id(self, session_id):
        self._tg_session_id = session_id
        self.project_config.set_tg_session_id(session_id, self.instance)

    @property
    def imex_file(self):
        return os.path.join(self.project_config.projectpath, "config.imex")

    @property
    def tg_project_id(self):
        if not self._tg_project_id:
            self._tg_project_id = self.project_config.get_tg_project_id(
                self.instance
            )
        return self._tg_project_id

    @tg_project_id.setter
    def tg_project_id(self, project_id):
        self._tg_project_id = project_id
        self.project_config.set_tg_project_id(project_id, self.instance)

    @property
    def tg_client(self):
        if not self._tg_client:
            self._tg_client = TGclient(self.tg_session_id, self.instance)
        return self._tg_client

    def create_tg_project(self, name, description=""):
        # def create_tg_project(self, name, instance="test", description=""):
        if not self.tg_session_id:
            return []
        tg_project_id = self.tg_client.create_project(name, description)
        if tg_project_id:
            self.tg_project_id = tg_project_id

    def clear_tg_project(self, tg_project_id):
        # def clear_tg_project(self, tg_project_id, instance="test"):
        if not self.tg_session_id:
            return False
        # clear contents of tg-project at textgrid server
        self.tg_client.clear_project(tg_project_id)

        # remove imex files locally
        for c in self.project.collections.values():
            c["modeler"].remove_imex_file()
        self.project_config.other_files.remove_imex_file()

        return True

    def delete_tg_project(self, tg_project_id):
        # def delete_tg_project(self, tg_project_id, instance="test"):
        if not self.tg_session_id:
            return False
        # delete tg-project at textgrid server
        res = self.tg_client.delete_project(tg_project_id)
        # delete tg-project at local config if successful AND
        # is the currently defined project_id
        if res and tg_project_id == self.tg_project_id:
            self.tg_project_id = None
        return True

    def get_tg_projects(self):
        # def get_tg_projects(self, instance="test"):
        if not self.tg_session_id:
            return []
        return self.tg_client.get_assigned_projects()

    def get_tg_project(self, project_id):
        if self.tg_session_id:
            return self.tg_client.get_project_description(project_id)

    def upload_tg_project(self):
        # def upload_tg_project(self, instance="test"):
        if not self.tg_session_id:
            return False

        for c_name in self.collections:
            collection = self.collections[c_name]
            # step 1: create required metadata
            collection["modeler"].render_collection()
            # step 2: upload collection to textgrid server
            collection["modeler"].upload(
                tg_session_id=self.tg_session_id,
                tg_project_id=self.tg_project_id,
                tg_server=self.instance,
            )

    def upload_collection(self, collection_name):
        # def upload_collection(self, collection_name, instance="test"):
        if not self.tg_session_id:
            return False
        log.info("Uploading collection (%s)..." % collection_name)
        collection = self.project.get_collection(collection_name)
        # step 1: create required metadata
        collection["modeler"].render_collection()
        # step 2: upload collection to textgrid server
        collection["modeler"].upload(
            tg_session_id=self.tg_session_id,
            tg_project_id=self.tg_project_id,
            tg_server=self.instance,
        )

    def upload_other_files(self):
        # def upload_other_files(self, instance="test"):
        if not self.tg_session_id:
            return False
        log.info("Uploading other files...")
        self.project_config.other_files.upload(
            tg_session_id=self.tg_session_id,
            tg_project_id=self.tg_project_id,
            tg_server=self.instance,
        )

    def get_project_contents(self):
        return self.tg_client.get_project_contents(self.tg_project_id)

    def publish_project(self):
        # def publish_project(self, instance="test"):
        if not self.tg_session_id:
            return False
        log.info("Publishing project...")
        self.tg_client.publish(self.tg_project_id)


class CollectionParser(object):

    def __init__(self, collection):
        self.collection = collection
        self._elements = None

    @property
    def elements(self):
        if self._elements is None:
            self._elements = []
            for file in self.collection.elements:
                self._elements.append(
                    {
                        "file": file,
                        "tei_parser": TEIParser(fullpath=file["fullpath"]),
                    }
                )
        return self._elements

    def test_xpath(self, xpath):
        results = []
        for element in self.elements:
            result = element["tei_parser"].find(xpath)
            if result is not None:
                results.append(
                    {
                        "filename": element["file"]["filename"],
                        "result": result,
                    }
                )
        return {
            "results": results,
            "count": {
                "total": len(self.collection.elements),
                "found": len(results),
                "percentage": round(
                    len(results) / len(self.collection.elements) * 100
                ),
            },
        }



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

import click
from .util import cli_startup

log = logging.getLogger(__name__)


@click.group()
@click.option("--debug/--no-debug", "-d", is_flag=True, default=False)
@click.option("--login", default="rk")
@click.pass_context
def main(ctx, debug, login):
    cli_startup(log_level=debug and logging.DEBUG or logging.INFO)
    ctx.ensure_object(dict)
    ctx.obj["DEBUG"] = debug
    ctx.obj["login"] = login



# -*- coding: utf-8 -*-
# Copyright (C) 2023 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.deimport logging

import logging

from .config import LOG_LEVEL

log = logging.getLogger(__name__)

log_config = dict(
    level=LOG_LEVEL,
    format="%(asctime)s %(name)-10s %(levelname)-4s %(message)s",
)

logging.basicConfig(**log_config)
logging.getLogger("").setLevel(LOG_LEVEL)



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

from flask import Blueprint, render_template, request  # type: ignore

# from flask_login import login_required  # type: ignore

from tgp_backend.project import Project

log = logging.getLogger(__name__)

main_views = Blueprint("views", __name__)


# favored route
@main_views.route("/overview", methods=["GET", "POST"])
# additional routes for compatibility
@main_views.route("/", methods=["GET"])
@main_views.route("/project", methods=["GET", "POST"])
@main_views.route("/projects", methods=["GET", "POST"])
def overview():
    return render_template(
        "projects_overview.html", sub_title="Projects Overview"
    )


@main_views.route("/project/<string:projectname>", methods=["GET", "POST"])
def project_view(projectname=None):
    return render_template(
        "project_mainview.html",
        current_project=projectname,
        project=Project(projectname),
        tab=request.args.get("tab"),
    )


@main_views.route("/modal/container", methods=["GET"])
def modal_container():
    """
    Render a modal spinner template.
    This is used to show a loading spinner while processing requests.
    """
    return render_template("modal/empty_container.html")


@main_views.route("/tabs/container", methods=["GET"])
def tab_container():
    """
    Render a tab spinner template.
    This is used to show a loading spinner while processing requests.
    """
    return render_template("tabs/empty_container.html")


@main_views.route("/details/container", methods=["GET"])
def details_container():
    """
    Render a details spinner template.
    This is used to show a loading spinner while processing requests.
    """
    return render_template("details/empty_container.html", show_spinner=True)



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

from flask import Blueprint, render_template, request

from tgp_backend.project import Project

log = logging.getLogger(__name__)

publication_routes = Blueprint("publication", __name__)


@publication_routes.route(
    "/tab-prepare-upload/<string:projectname>", methods=["GET"]
)
def tab_upload(projectname):
    project = Project(projectname)
    return render_template(
        "tabs/upload.html",
        project=project,
    )


@publication_routes.route(
    "/tab-check-result/<string:projectname>", methods=["GET"]
)
def tab_check_result(projectname):
    project = Project(projectname)

    images = [
        {
            "filename": "01_title.png",
            "alt": "Confirm Title",
            "caption": "Confirm your project's title and short description",
            "icon": "bi-card-heading text-primary",
        },
        {
            "filename": "02_avatar.png",
            "alt": "Check Avatar Image",
            "caption": "Check that your avatar image looks correct.",
            "icon": "bi-person-circle text-secondary",
        },
        {
            "filename": "03_readme.png",
            "alt": "Check README.md",
            "caption": "Check your README.md at your landing page.",
            "icon": "bi-file-earmark-text text-success",
        },
        {
            "filename": "04_count_objects.png",
            "alt": "Check Number of Imported Objects",
            "caption": "Confirm that the number of imported objects — especially XML files and collections — is correct.",
            "icon": "bi-123 text-info",
        },
        {
            "filename": "05_facets.png",
            "alt": "Check Facets and Metadata",
            "caption": "Ensure that all facets and metadata are displayed correctly.",
            "icon": "bi-ui-checks-grid text-warning",
        },
        {
            "filename": "06_transformation.png",
            "alt": "Check TEI to HTML Transformation",
            "caption": "Check that some of your TEI files are being correctly transformed into HTML.",
            "icon": "bi-filetype-html text-danger",
        },
        # {"filename": "", "alt": "", "caption": ""},
    ]

    return render_template(
        "tabs/check_upload.html",
        project=project,
        images=images,
    )


@publication_routes.route(
    "/tab-publication/<string:projectname>", methods=["GET"]
)
def tab_publication(projectname):
    project = Project(projectname)
    return render_template(
        "tabs/publication.html",
        project=project,
    )


class TGProjectHandler:
    def __init__(self, projectname, instance, action="upload"):
        self.project = Project(projectname)
        self.tg_project = self.project.get_tgp(instance)
        self.instance = instance
        self.action = action

    def _render_template(self, action="upload", **kwargs):
        return render_template(
            f"includes/{self.action}_form.html",
            project=self.project,
            tg_project=self.tg_project,
            instance=self.instance,
        )

    def save_session_id(self, session_id):
        self.tg_project.tg_session_id = session_id
        return self._render_template()

    def save_tg_project_id(self, tg_project_id):
        self.tg_project.tg_project_id = tg_project_id
        return self._render_template()

    def delete_tg_project(self, tg_project_id):
        self.tg_project.delete_tg_project(tg_project_id)
        return self._render_template()

    def clear_tg_project(self, tg_project_id):
        self.tg_project.clear_tg_project(tg_project_id)
        return self._render_template()

    def create_tg_project(self, tg_projectname):
        self.tg_project.create_tg_project(tg_projectname)
        return self._render_template()

    def upload_tg_project(self):
        self.tg_project.upload_tg_project()
        return self._render_template()

    def upload_collection(self, collection_name):
        self.tg_project.upload_collection(collection_name=collection_name)
        return self._render_template()

    def upload_other_files(self):
        self.tg_project.upload_other_files()
        return self._render_template()

    def publish_project(self):
        self.tg_project.publish_project()
        return self._render_template()


@publication_routes.route(
    "/save_session_id/<string:projectname>/<string:instance>/<string:action>",
    methods=["POST"],
)
def save_session_id(projectname, instance, action):
    return TGProjectHandler(projectname, instance, action).save_session_id(
        request.form.get("tg_auth_session_id")
    )


@publication_routes.route(
    "/save_tg_project_id/<string:projectname>/<string:instance>/<string:tg_project_id>/<string:action>",
    methods=["POST"],
)
def save_tg_project_id(projectname, instance, tg_project_id, action):
    return TGProjectHandler(projectname, instance, action).save_tg_project_id(
        tg_project_id
    )


@publication_routes.route(
    "/delete_tg_project_id/<string:projectname>/<string:instance>/<string:tg_project_id>/<string:action>",
    methods=["POST"],
)
def delete_tg_project_id(projectname, instance, tg_project_id, action):
    return TGProjectHandler(projectname, instance, action).delete_tg_project(
        tg_project_id
    )


@publication_routes.route(
    "/clear_tg_project/<string:projectname>/<string:instance>/<string:tg_project_id>/<string:action>",
    methods=["POST"],
)
def clear_tg_project(projectname, instance, tg_project_id, action):
    return TGProjectHandler(projectname, instance, action).clear_tg_project(
        tg_project_id
    )


@publication_routes.route(
    "/create_tg_project/<string:projectname>/<string:instance>/<string:action>",
    methods=["POST"],
)
def create_tg_project(projectname, instance, action):
    return TGProjectHandler(projectname, instance, action).create_tg_project(
        request.form.get("tg_projectname")
    )


@publication_routes.route(
    "/upload_project/<string:projectname>/<string:instance>/<string:action>",
    methods=["POST"],
)
def upload_project(projectname, instance, action):
    return TGProjectHandler(projectname, instance, action).upload_tg_project()


@publication_routes.route(
    "/upload_collection/<string:projectname>/<string:collection>/<string:instance>/<string:action>",
    methods=["POST"],
)
def upload_collection(projectname, collection, instance, action):
    return TGProjectHandler(projectname, instance, action).upload_collection(
        collection
    )


@publication_routes.route(
    "/upload_other_files/<string:projectname>/<string:instance>/<string:action>",
    methods=["POST"],
)
def upload_other_files(projectname, instance, action):
    return TGProjectHandler(projectname, instance, action).upload_other_files()


@publication_routes.route(
    "/publish_project/<string:projectname>/<string:instance>/<string:action>",
    methods=["POST"],
)
def publish_project(projectname, instance, action):
    return TGProjectHandler(projectname, instance, action).publish_project()



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2026 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

from flask import Blueprint, session
from flask_json import json_response

log = logging.getLogger(__name__)

tab_manager = Blueprint("tab_manager", __name__)


@tab_manager.route("/get_last_tab/<string:projectname>", methods=["GET"])
def get_last_tab(projectname):
    tabname = session.get(f"last_tab_{projectname}", None)
    return json_response(status=200, initial_tab=tabname or "tab-edit-project")


@tab_manager.route(
    "/set_last_tab/<string:projectname>/<string:tabname>", methods=["POST"]
)
def set_last_tab(projectname, tabname):
    if tabname:
        session[f"last_tab_{projectname}"] = tabname
        return json_response(
            response="OK",
            message=f"Last tab for project '{projectname}' set to '{tabname}'",
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

import os

from flask import Blueprint, render_template, request, send_file
from flask_json import json_response

from io import BytesIO

from tg_model.tei import TEIParser

from tgp_backend.directories import generateList
from tgp_backend.project import Project
from tgp_backend.util import parse_request_data

log = logging.getLogger(__name__)

collection_routes = Blueprint("collection", __name__)


# ***TABS***
# **********
@collection_routes.route(
    "/tab_select_directories/<string:projectname>", methods=["GET", "POST"]
)
def tab_select_directories(projectname):

    project = Project(projectname)
    if request.method == "POST":
        project.project_config.update(
            tei_directories=request.form.getlist("selected_folder")
        )

    tei_directories = [
        sp["inpath"] for sp in project.project_config.get_subprojects()
    ]

    return render_template(
        "tabs/select_directories.html",
        project=project,
        selected_directories=tei_directories,
    )


@collection_routes.route(
    "/tab_manage_collections/<string:projectname>", methods=["GET"]
)
def tab_manage_collections(projectname):

    project = Project(projectname)

    return render_template(
        "tabs/manage_collections.html",
        project=project,
    )


@collection_routes.route(
    "/tab_validate_metadata/<string:projectname>", methods=["GET"]
)
@collection_routes.route(
    "/tab_validate_metadata/<string:projectname>/<string:refresh>",
    methods=["GET"],
)
def tab_validate_metadata(projectname, refresh=True):

    project = Project(projectname)
    return render_template(
        "tabs/validate_metadata.html",
        projectname=projectname,
        validation_results=project.get_validation_results(refresh=refresh),
    )


# ***OTHER***
# **********
@collection_routes.route(
    "/load_collection/<string:projectname>/<string:collectionname>",
    methods=["GET"],
)
def load_collection(projectname, collectionname):
    collection = None
    project = Project(projectname)
    collection = None
    if collectionname:
        collection = project.get_collection(collectionname)

    return render_template(
        "details/manage_collection.html",
        collectionname=collectionname,
        collection=collection["config"],
        project=project,
    )


@collection_routes.route(
    "/save_collection_attributes/<string:projectname>/<string:collectionname>",
    methods=["POST"],
)
def save_collection_attributes(projectname, collectionname):
    project = Project(projectname)
    collection = project.get_collection(collectionname)
    collection_config = collection["config"]

    multi_attribs = ["collector", "rights_holder"]
    for multi_attrib in multi_attribs:
        elements = []
        for name, url in zip(
            request.form.getlist(f"{multi_attrib}_name"),
            request.form.getlist(f"{multi_attrib}_url"),
        ):
            elements.append({"fullname": name, "url": url})
        setattr(collection_config, multi_attrib, elements)

    for attrib in ["short_title", "long_title"]:
        value = request.form.get(attrib)
        setattr(collection_config, attrib, value)

    collection_config.save()
    return load_collection(
        projectname=projectname, collectionname=collectionname
    )


@collection_routes.route(
    "/save_collection_classifications/<string:projectname>/<string:collectionname>",
    methods=["POST"],
)
def save_collection_classifications(projectname, collectionname):
    project = Project(projectname)
    collection = project.get_collection(collectionname)
    collection_config = collection["config"]

    # Save basic classifications...
    collection_config.basic_classifications = parse_request_data(
        request, "basic_classifications"
    )
    # ...and GND subjects
    collection_config.gnd_subjects = parse_request_data(
        request, "gnd_subjects"
    )

    collection_config.save()
    return load_collection(
        projectname=projectname, collectionname=collectionname
    )


@collection_routes.route(
    "/save_collection_metadata/<string:projectname>/<string:collectionname>",
    methods=["POST"],
)
def save_collection_metadata(projectname, collectionname):
    project = Project(projectname)
    collection = project.get_collection(collectionname)
    collection_config = collection["config"]
    for attrib in request.form:
        if "xpath" in attrib:
            key = attrib.replace("_xpath", "")
            type_ = "xpath"
        elif "value" in attrib:
            key = attrib.replace("_value", "")
            type_ = "value"
        if key in collection_config.xpath_or_value_attribs:
            cc_attrib = getattr(collection_config, key)
            cc_attrib[type_] = request.form.get(attrib)
        else:
            log.warning(
                f"Attribute '{attrib}' not found in collection config."
            )
    collection_config.save()
    return load_collection(
        projectname=projectname, collectionname=collectionname
    )


@collection_routes.route(
    "/download_collection_yaml/<string:projectname>/<string:collectionname>",
    methods=["GET"],
)
def download_collection_yaml(projectname, collectionname):
    project = Project(projectname)
    collection = project.get_collection(collectionname)
    return send_file(
        collection["config"].path,
        mimetype="application/yaml",
        as_attachment=True,
        download_name=f"{collectionname}.yaml",
    )


@collection_routes.route(
    "/upload_collection_yaml/<string:projectname>/<string:collectionname>",
    methods=["POST"],
)
def upload_collection_yaml(projectname, collectionname):
    collection = Project(projectname).get_collection(collectionname)

    # Get the uploaded file and save it to the collection config path
    new_config_file = request.files.get("config")
    new_config_file.save(collection["config"].path)

    # Reload the collection config
    new_collection = Project(projectname).get_collection(collectionname)

    # Preserve short and long title
    for key in ["short_title", "long_title"]:
        setattr(
            new_collection["config"], key, getattr(collection["config"], key)
        )
    new_collection["config"].save()

    # reload the collection view
    return load_collection(
        projectname=projectname, collectionname=collectionname
    )


@collection_routes.route(
    "/modal/tei_explorer/<string:projectname>/<string:collectionname>",
    methods=["GET", "POST"],
)
def modal_tei_explorer(projectname, collectionname):
    project = Project(projectname)
    # collection = project.get_collection(collectionname)
    dir_list_dict, file_list_dict = generateList(
        project.get_subproject_inpath(collectionname)
    )

    xpath = request.args.get("xpath", "")

    return render_template(
        "modal/tei_explorer.html",
        projectname=projectname,
        collectionname=collectionname,
        dir_list=dir_list_dict,
        file_list=file_list_dict,
        xpath=xpath,
        project=project,
    )


@collection_routes.route("/load_tei_content", methods=["GET"])
def load_tei_content():
    path = request.args.get("path")
    log.debug("load_tei_content path: %s" % path)
    _type = request.args.get("type")
    if path is not None:
        tei_parser = TEIParser(fullpath=path)
        if _type == "header":
            return json_response(
                value="OK",
                content=tei_parser.find(
                    "//teiHeader", node_as_text=True
                ).decode("utf-8"),
            )
        elif _type == "text":
            return json_response(
                value="OK",
                content=tei_parser.find(".//text", node_as_text=True).decode(
                    "utf-8"
                ),
            )
        return json_response(value="Unknown type requested!")


@collection_routes.route(
    "/modal/xpath_parser/<string:projectname>/<string:collectionname>",
    methods=["GET", "POST"],
)
def modal_xpath_parser(
    projectname,
    collectionname,
):
    project = Project(projectname)
    collection = project.get_collection(
        collectionname,
    )
    collection_parser = collection["parser"]

    xpath = request.args.get("xpath")
    if not xpath:
        xpath = request.form.get("xpath", "")

    return render_template(
        "modal/xpath_parser.html",
        xpath=xpath,
        projectname=projectname,
        collectionname=collectionname,
        collection_parser=collection_parser,
    )


@collection_routes.route("/export_tsv/<projectname>/<collectionname>")
def export_tsv(projectname, collectionname):
    project = Project(projectname)
    collection = project.get_collection(
        collectionname,
    )
    filepath = collection["modeler"].export()
    if os.path.isfile(filepath):
        with open(filepath, "rb") as file:
            file_content = BytesIO(file.read())

        return send_file(
            file_content,
            download_name=os.path.basename(filepath),
            as_attachment=True,
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

import click  # type: ignore
import os

from flask import Flask, render_template, request, session, url_for  # type: ignore
from flask_json import FlaskJSON  # type: ignore

from tgp_backend.project import Project

from tgp_backend.auth import SecretKeyManager
from tgp_backend.config import (
    LOG_LEVEL,
    PATH_PROJECTS,
    get_config_value,
)
from tgp_backend.nextcloud import Nextcloud

# from tgp_backend.user import UserManager

from .routes.project import project_routes
from .routes.views import main_views
from .routes.data import data_routes
from .routes.tabs import tab_manager
from .routes.collection import collection_routes
from .routes.publication import publication_routes

log = logging.getLogger(__name__)

log_level = get_config_value("log", "level", default="DEBUG") == "DEBUG"
log.setLevel(log_level)
file_handler = logging.FileHandler(
    get_config_value("log", "path", default="/tmp/tg_prepare.log")
)
file_handler.setLevel(log_level)
logging.getLogger().addHandler(file_handler)

base_params = {
    "title": "TG Prepare",
}

app = Flask(__name__)
FlaskJSON(app)

secret_manager = SecretKeyManager(PATH_PROJECTS)
app.secret_key = secret_manager.secret_key

# Additional security settings, if not in DEBUG mode
app.config.update(SESSION_COOKIE_NAME="tgp_session")
if LOG_LEVEL != "DEBUG":
    app.config.update(
        SESSION_COOKIE_SECURE=True,  # Only allow cookies over HTTPS
        SESSION_COOKIE_HTTPONLY=True,  # Prevent JavaScript access to cookies
        SESSION_COOKIE_SAMESITE="Strict",  # Protect against CSRF attacks
    )

# Initialize Blueprints
app.register_blueprint(main_views)
app.register_blueprint(project_routes)
app.register_blueprint(data_routes)
app.register_blueprint(tab_manager)
app.register_blueprint(collection_routes)
app.register_blueprint(publication_routes)


def get_textgrid_login_url(instance):
    """
    Returns the TextGrid login URL based on the instance (test or production).
    """
    if instance == "test":
        return "https://test.textgridlab.org/1.0/Shibboleth.sso/Login?target=/1.0/secure/TextGrid-WebAuth.php?authZinstance=test.textgridlab.org"
    else:
        return "https://textgridlab.org/1.0/Shibboleth.sso/Login?target=/1.0/secure/TextGrid-WebAuth.php?authZinstance=textgrid-esx2.gwdg.de"


def get_prefix():
    return request.headers.get(
        "X-Forwarded-Prefix", request.headers.get("X-Script-Name", "")
    )


# URL-For Override mit Context Processor
@app.context_processor
def override_url_for():
    return dict(
        url_for=lambda *args, **kwargs: get_prefix() + url_for(*args, **kwargs)
    )


def get_projects():
    projects = []
    for sub in os.listdir(PATH_PROJECTS):
        projectpath = "%s/%s" % (PATH_PROJECTS, sub)
        if os.path.isdir(projectpath):
            projects.append(Project(sub))
    return sorted(projects, key=lambda x: x.name.lower())


app.jinja_env.globals.update(
    len=len,
    round=round,
    title="TG Prepare",
    get_projects=get_projects,
    get_textgrid_login_url=get_textgrid_login_url,
)


def _startup():
    logging.getLogger("zeep").setLevel(logging.INFO)
    app.run(
        host=get_config_value("main", "host", default="0.0.0.0"),
        port=get_config_value("main", "port", default=8077),
        debug=get_config_value("log", "level", default="DEBUG") == "DEBUG",
    )


@click.command()
@click.option("--path", "-p", default=None)
def startup(path):
    base_params["path"] = path if path else os.getcwd()
    _startup()


@app.route("/nextcloud_tab/", methods=["POST"])
def nextcloud_tab():
    nextcloud = Nextcloud(**session)
    return render_template(
        "nxc_tab.html",
        nextcloud=nextcloud if nextcloud.test_connection() else None,
        user=session.get("username", "-"),
    )


if __name__ == "__main__":
    startup()



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de
import logging

from tgadmin.tgadmin import _crud_delete_op
from tgadmin.tgimport import TGimport

from tgclients import (
    TextgridAuth,
    TextgridConfig,
    TextgridCrud,
    TextgridSearch,
)
from tgclients.config import PROD_SERVER, TEST_SERVER

log = logging.getLogger(__name__)


class TGclient(object):
    def __init__(self, sid, instance="test", verbose=False):
        self.sid = sid

        if instance == "live":
            self.server = PROD_SERVER
        else:
            self.server = TEST_SERVER
        self.config = TextgridConfig(self.server)

        self.crud = TextgridCrud(self.config)
        self.tgauth = TextgridAuth(self.config)
        self.tgsearch = TextgridSearch(self.config, nonpublic=True)

        self._contents = {}

    def check_session(self):
        try:
            if self.sid:
                self.tgsearch.search(sid=self.sid)
                return True
        except Exception as e:
            log.error(f"Error checking session id: {self.sid} : {e}")
        return False
        # return self.tgauth.check_session(self.sid)  # returns username

    def create_project(self, name, description=""):
        log.info(f"Creating project {name}")
        return self.tgauth.create_project(self.sid, name, description)

    def clear_project(self, project_id):
        # delete content of project
        # repeat until whole content has been deleted
        # (necessary because of default limit in '_crud_delete_op')
        log.info(f"Clearing project contents {project_id}")
        content = self.get_project_content(project_id)
        handled_count = 0
        initial_count = int(content.hits)
        while handled_count < initial_count:
            for tgobj in content.result:
                # can only delete objects that are not already published
                if tgobj.object_value.generic.generated.availability == None:
                    _crud_delete_op(self, tgobj)
                handled_count += 1
            content = self.get_project_content(project_id)

    def delete_project(self, project_id):
        self.clear_project(project_id)

        log.info(f"Deleting project {project_id}")
        return self.tgauth.delete_project(self.sid, project_id)

    def get_project_content(self, project_id):
        log.warning("Deprecated: use TGclient.get_project_contents() instead")
        contents = self.tgsearch.search(
            filters=["project.id:" + project_id], sid=self.sid
        )
        return contents

    def _query_contents(self, tg_project_id):
        if not self._contents.get(tg_project_id):
            log.debug(f"Querying contents for project id: {tg_project_id}")
            try:
                self._contents[tg_project_id] = self.tgsearch.search(
                    filters=["project.id:" + tg_project_id],
                    sid=self.sid,
                )
            except Exception as e:
                log.error(f"Error querying contents for project: {e}")
                return []

        return self._contents[tg_project_id].result

    def get_project_contents(
        self,
        tg_project_id,
        public=None,
        count=False,
        category=None,
    ):
        _filter = None
        if category == "tei":
            _filter = ["text/xml"]
        elif category == "images":
            _filter = ["image/jpeg", "image/png", "image/gif"]
        elif category == "other":
            _filter = [
                "image/png",
                "text/markdown",
                "application/xslt+xml",
                "text/tg.portalconfig+xml",
            ]

        contents = self._query_contents(tg_project_id)
        results = {}
        for tgobj in contents:
            result = {}
            result["title"] = tgobj.object_value.generic.provided.title[0]
            result["tguri"] = (
                tgobj.object_value.generic.generated.textgrid_uri.value
            )
            result["mime"] = tgobj.object_value.generic.provided.format
            result["published"] = (
                tgobj.object_value.generic.generated.availability is not None
            )
            log.debug(f"Found object: {result}")

            if _filter is None or result["mime"] in _filter:
                results[result["title"]] = result
            else:
                log.info(
                    "Skipping object with unsupported mime-type: %s"
                    % result["mime"]
                )
        return len(results) if count else results

    def get_project_description(self, project_id):
        desc = self.tgauth.get_project_description(project_id)
        if desc:
            return {
                "id": project_id,
                "name": desc.name,
                "description": desc.description,
                "tei_count": self.get_project_contents(
                    project_id, count=True, category="tei"
                ),
                "img_count": self.get_project_contents(
                    project_id, count=True, category="images"
                ),
                "other_count": self.get_project_contents(
                    project_id, count=True, category="other"
                ),
                "content_count": self.get_project_contents(
                    project_id, count=True
                ),
            }
        else:
            log.warning(f"Cannot find project description for: {project_id}")

    def get_assigned_projects(self):
        log.info("Listing assigned projects")
        try:
            _projects = self.tgauth.list_assigned_projects(self.sid)
        except Exception as e:
            log.error(f"Error listing assigned projects: {e}")
            return []

        for project_id in reversed(_projects):
            yield self.get_project_description(project_id)

    # def upload(self, filenames: list[str], project_id: str, imex_file: str):
    #     tg_importer = TGimport(
    #         self.sid,
    #         self.crud,
    #         project_id=project_id,
    #         ignore_warnings=True,
    #         imex_location=imex_file,
    #     )
    #     tg_importer.upload(filenames=filenames, threaded=True)

    def publish(self, tg_project_id):
        log.info(f"Publishing project {tg_project_id}")
        # TODO: implement publish logic in tgclients
        # see tgadmin.tgadmin.publish() tgadmin.py::Z646
        breakpoint()



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

import os
import shutil

from flask import Blueprint, redirect, render_template, request
from flask import url_for as default_url_for
from flask_json import json_response


from tgp_backend.config import PATH_PROJECTS
from tgp_backend.project import Project

from .views import overview

log = logging.getLogger(__name__)

project_routes = Blueprint("project", __name__)


def get_prefix():
    return request.headers.get(
        "X-Forwarded-Prefix", request.headers.get("X-Script-Name", "")
    )


def url_for(*args, **kwargs):
    """Overrides Flask's url_for globally to include the prefix"""
    return get_prefix() + default_url_for(*args, **kwargs)


@project_routes.route("/new_project", methods=["POST"])
def new_project():
    project = Project(request.form.get("projectname"))
    project.create()
    return redirect(url_for("views.project_view", projectname=project.path))


@project_routes.route("/delete_project/<string:projectname>", methods=["POST"])
def delete_project(projectname):

    # projectname = request.form.get("projectname")
    fullpath = f"{PATH_PROJECTS}/{projectname}"

    # Delete the project
    if fullpath.strip("/") == PATH_PROJECTS.strip("/"):
        log.error(f"Cannot delete main path ({PATH_PROJECTS})!")
    elif os.path.exists(fullpath):
        shutil.rmtree(fullpath)
    else:
        log.warning("Project does not exist!")

    return overview()


@project_routes.route(
    "/modal_delete_project/<string:projectname>", methods=["POST"]
)
def modal_delete_project(projectname):
    return render_template(
        "modal/delete_project.html",
        projectname=projectname,
    )


@project_routes.route(
    "/tab-edit-project/<string:projectname>", methods=["GET", "POST"]
)
def tab_edit_project(projectname):
    project = Project(projectname)
    if request.method == "POST":
        collectors = []
        for name, url in zip(
            request.form.getlist("collector_name"),
            request.form.getlist("collector_url"),
        ):
            collectors.append({"name": name, "url": url})

        project.update(
            title=request.form["title"],
            description=request.form["description"],
            collectors=collectors,
            avatar=request.files["avatar"],
            xslt=request.files.get("xslt"),
        )
    return render_template(
        "tabs/edit_project.html",
        project=project,
    )


@project_routes.route("/delete-xslt/<string:projectname>", methods=["DELETE"])
def delete_xslt(projectname):
    project = Project(projectname)
    log.debug(f"Deleting XSLT for project {projectname}")
    project.clear_xslt()
    return json_response("OK")



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2024 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

from setuptools import setup, find_packages

setup(
    name="tg_prepare",
    version="2.2.9",
    description="Simple UI to handle TextGrid imports visually.",
    author="Ralf Klammer, Moritz Wilhelm",
    author_email="ralf.klammer@tu-dresden.de, moritz.wilhelm@tu-dresden.de",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "click",
        "flask",
        "flask_json",
        "flask_login",
        "tg_model>=3.11.2",
        "tgclients",
        "tgadmin",
        "nextcloud-api-wrapper",
    ],
    package_data={
        "tgp_ui": [
            "static/js/*.js",
            "static/css/*.css",
            "static/svg/*.svg",
            "templates/*.html",
        ],
    },
    entry_points={
        "console_scripts": [
            "tgp_cli = tgp_backend.cli:main",
            "tgp_app = tgp_ui.app:startup",
        ]
    },
)



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2025 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
# moritz.wilhelm@tu-dresden.de

import logging

log = logging.getLogger(__name__)


class SessionManager:
    def __init__(self, session):
        self.session = session


class SessionManagerNextcloud(SessionManager):

    def save_credentials(self, form_data):
        log.info("Saving credentials to session")
        self.session["nextcloud"] = {
            "username": form_data.get("nextcloud_user"),
            "password": form_data.get("nextcloud_password"),
            "url": form_data.get("nextcloud_url"),
            "folder": form_data.get("nextcloud_folder", ""),
        }

    def get_credentials(self):
        log.info("Retrieving credentials from session")
        return self.session.get("nextcloud", {})

    def delete_nextcloud_credentials(self):
        log.info("Deleting credentials from session")
        self.session.pop("nextcloud", None)

    def request_has_valid_credentials(self, request):
        """
        Check if the request contains valid Nextcloud credentials.
        """
        relevant_form_keys = [
            "nextcloud_url",
            "nextcloud_user",
            "nextcloud_password",
        ]
        relevant_form_values = [
            request.form.get(key) for key in relevant_form_keys
        ]
        return all(relevant_form_values)



# -*- coding: utf-8 -*-
# Copyright (C) 2023-2024 TU-Dresden (ZIH)
# ralf.klammer@tu-dresden.de
import logging

import os

from configparser import ConfigParser

log = logging.getLogger(__name__)


def get_config_value(section, parameter, default=None):
    _config = ConfigParser()
    _config.read("config.ini")

    if section not in _config:
        log.warn("Section: %s not in *.ini -> using default" % section)
        return default
    config_val = _config[section].get(parameter)
    if not config_val:
        log.info(
            "Parameter: %s not in section: (%s) of *.ini -> using default: %s"
            % (parameter, section, default)
        )
        return default
    else:
        return config_val


LOG_LEVEL = get_config_value("log", "level", default="INFO")
MAX_FILENAME_LENGTH = get_config_value("main", "maxFileNameLength", default=15)
PATH_PROJECTS = get_config_value(
    "main", "path_projects", default=os.path.abspath(os.getcwd() + "/projects")
)
PATH_DATA = get_config_value(
    "main", "path_data", default=os.path.abspath(os.getcwd() + "/data")
)
# Create the projects directory if it does not exist
if not os.path.exists(PATH_PROJECTS):
    os.makedirs(PATH_PROJECTS)

# Create the data directory if it does not exist
if not os.path.exists(PATH_DATA):
    os.makedirs(PATH_DATA)
